@@ -1,24 +1,18 @@
 from __future__ import annotations
 
-from dataclasses import Field, asdict, dataclass, field, fields, is_dataclass
+from dataclasses import MISSING, Field, asdict, dataclass, field, fields, is_dataclass
 from types import ModuleType
-from typing import Any, Dict, List, Type, Union
+from typing import Any, Dict, List, Tuple, Type, Union
 
 from pydantic import create_model
 from pydantic.error_wrappers import ValidationError
 from pydantic.types import StrictBool, StrictBytes, StrictFloat, StrictInt, StrictStr
 
-from pytest_embrace.anno import AnnotationMap, DeriveFromFileName, get_pep_593_values
-
-# from . import anno
-from .case import CaseType, Trickle
+from .anno import AnnotationMap, get_pep_593_values
+from .case import CaseType, DeriveFromFileName, Trickle
 from .exc import CaseConfigurationError
 
 
-class Sentinel:
-    ...
-
-
 @dataclass
 class AttrInfo:
     dc_field: Field
@@ -38,6 +32,12 @@ PYDANTIC_STRICTIFICATION_MAP = {
     int: StrictInt,
     float: StrictFloat,
     bool: StrictBool,
+    # in dataclasses, Field.type is a string at times
+    "str": StrictStr,
+    "bytes": StrictBytes,
+    "int": StrictInt,
+    "float": StrictFloat,
+    "bool": StrictBool,
 }
 
 
@@ -57,7 +57,8 @@ class ModuleInfo:
         self.attrs: Dict[str, AttrInfo] = {}
 
         for field_ in spec:
-            if field_.name not in defined and len(anno_map) == 0:
+            field_meta = field_.metadata
+            if field_.name not in defined and len(anno_map) == 0 == len(field_meta):
                 continue
 
             if field_.name == "table":
@@ -71,22 +72,12 @@ class ModuleInfo:
                 pass
 
             anno_info = anno_map.get(field_.name)
-            if anno_info is None:
-                module_value = getattr(mod, field_.name)
-            else:
-                derive_name = next(
-                    (
-                        v
-                        for v in anno_info.annotations
-                        if isinstance(v, DeriveFromFileName)
-                    ),
-                    None,
-                )
+            if "derive_from_filename" in field_meta:
+                derive_name = field_meta["derive_from_filename"]
                 if derive_name is not None:
                     module_value = derive_name.get_attr_value(mod.__name__)
-                else:
-                    foo = f"{anno_info.annotations} {mod} {spec}"
-                    raise SyntaxError(foo)
+            elif anno_info is None:
+                module_value = getattr(mod, field_.name)
 
             attr_info = AttrInfo(field_, module_value)
             self.attrs[attr_info.name] = attr_info
@@ -146,8 +137,16 @@ class PydanticConfig:
 def revalidate_dataclass(case: CaseType, *, alias: str) -> CaseType:
     _raise_non_dataclass(case)
     kwargs = asdict(case)
-    validator_kwargs: Dict[str, Any] = {
-        k: (_strictify(type(v)), v) for k, v in kwargs.items()
+    validator_kwargs: Dict[str, Tuple[Any, Any]] = {
+        field.name: (
+            _strictify(field.type),
+            field.default
+            if field.default is not MISSING
+            else (
+                field.default_factory() if field.default_factory is not MISSING else ...
+            ),
+        )
+        for field in fields(case)
     }
 
     Validator = create_model(
@@ -169,24 +168,52 @@ def from_trickling_module(cls: Type[CaseType], module: ModuleType) -> List[CaseT
     assert table is not None
 
     UNSET = object()
-    trickle_attr_defaults = {
-        attr: (trickle_instance, getattr(module, attr, UNSET))
-        for attr in dir(cls)
-        if not attr.startswith("__")
-        and isinstance((trickle_instance := getattr(cls, attr)), Trickle)
+    # build up the default values set in the _module_
+    # for the attributes on _cls_ that were marked with trickles()
+    cls_fields = {field.name: field for field in fields(cls)}
+    values_from_filename = {
+        k: derive.get_attr_value(module.__name__)
+        for k, v in cls_fields.items()
+        if (derive := v.metadata.get("derive_from_filename")) is not None
     }
-    # keep track of table members who did not set a value for the trickles() attr
-    # when the attr was not set at the module elvel either. that's not ok!
+    trickle_attr_defaults: Dict[str, Tuple[Trickle, Any]] = {}
+    for attr in dir(module):
+        if attr.startswith("__"):
+            continue
+
+        config_from_cls = cls_fields.get(attr, UNSET)
+        field_meta = getattr(config_from_cls, "metadata", {})
+        value_from_module = getattr(module, attr, UNSET)
+        trickle_marker: Union[None, Trickle] = field_meta.get("trickle")
+
+        if trickle_marker is None:
+            continue
+
+        if value_from_module is UNSET:
+            continue
+
+        derive_from_filename: Union[DeriveFromFileName, None] = field_meta.get(
+            "derive_from_filename"
+        )
+        if derive_from_filename is None:
+
+            trickle_attr_defaults[attr] = trickle_marker, value_from_module
+        else:
+            trickle_attr_defaults[
+                attr
+            ] = trickle_marker, derive_from_filename.get_attr_value(module.__name__)
+
     unset_table_attrs_by_index: Dict[int, str] = {}
     illegal_overriders: Dict[int, str] = {}
 
     out: List[CaseType] = []
-    for i, case_ in enumerate(table):
-        case = revalidate_dataclass(case_, alias=f"{cls.__name__} #{i + 1}")
+    for i, case in enumerate(table):
         for k, v in asdict(case).items():
             trickle_config, trickled_down_val = trickle_attr_defaults.get(
-                k, (None, None)
+                k, (None, UNSET)
             )
+            if k in values_from_filename:
+                setattr(case, k, values_from_filename[k])
             if isinstance(v, Trickle):  # value is unset on the dataclass
                 if trickled_down_val is UNSET:
                     unset_table_attrs_by_index[i] = k
@@ -219,4 +246,16 @@ def from_trickling_module(cls: Type[CaseType], module: ModuleType) -> List[CaseT
         )
         raise CaseConfigurationError(error)
 
+    # do this at the _very_ end so the errors above have a chance to raise
+    # since validation errors don't capture the nuance of other table-related
+    # stuff that could have gone wrong already
+    for i, case in enumerate(out):
+        revalidate_dataclass(case, alias=f"table[{i}]:{case}")
     return out
+
+
+def load(cls: Type[CaseType], module: ModuleType) -> List[CaseType]:
+    if hasattr(module, "table"):
+        return from_trickling_module(cls, module)
+    else:
+        return [from_module(cls, module)]
